/* DSLR Box system
By Stephane BAUDOT
release 1.0
2013.04.28
*/
/* INCLUDES */
#include <LiquidCrystal.h>
#include <AnalogButtons.h>
#include <EEPROM.h>
#include <DSLR_Lightning.h>
#include <DSLR_Catcher.h>
#include <DSLR_Intervalometer.h>
#include "EEPROMAnything.h"
#include "MyTypedefStruct.h"
#include "MenuTree.h"

#define ANALOG_PIN	0
// Use the tool ANALOGButton_AdjustValue to adjust the value to your system
#define A0_BTRIGHT	0
#define A0_BTLEFT	179
#define A0_BTUP		472
#define A0_BTDOWN	306
#define A0_BTENTER	399

#define HMSMS_HMSMS 0	// 00h00m00s000ms
#define HMSMS_HMS   1	// 00h00m00s
#define HMSMS_SMS   2	// 00s000ms
#define HMSMS_HHPOS 0
#define HMSMS_MMPOS 1
#define HMSMS_SSPOS 2
#define HMSMS_MSPOS 3


///////////////////////////////////////////////////////////////////////////////
//                         --== PINS ALLOCATION ==--
// initialize the library with the numbers of the interface pins
///////////////////////////////////////////////////////////////////////////////
// Analog INPUT
	int aSensor1   = A1;                // pin for Photodiode 1
	int aSensor2   = A2;                // pin for Catcher
	int aSensor3   = A3;                // pin for Treshold
	
// PWB INPUT
// PWB OUTPUT
	const int ioCMD_Shutter		= 12;	// Command Shutter
	const int ioCMD_PreFocus	= 11;	// Command Pre-Focus
	const int ioCMD_Flash1		= 13;	// Command Flash

///////////////////////////////////////////////////////////////////////////////
//         --== LIQUID CRYSTAL MODULE VARIABLES ==--
// define variables & initialize default values
///////////////////////////////////////////////////////////////////////////////
	//LiquidCrystal lcd(rs, enable, d4, d5, d6, d7) // mode 4 bits - RW used
	LiquidCrystal   lcd(6,7,2,3,4,5);
	
///////////////////////////////////////////////////////////////////////////////
//                      --== BUTTONS VARIABLES ==--
// initialize the buttons advance functions                                  //
//   Default hold duration is 1 second, lets make it 5 seconds for ENTER
///////////////////////////////////////////////////////////////////////////////
	AnalogButtons anaBT(ANALOG_PIN, 100, &handleButtons);
	Button b1 = Button(BT_LEFT,		A0_BTLEFT,	A0_BTLEFT);
	Button b2 = Button(BT_RIGHT,	A0_BTRIGHT,	A0_BTRIGHT);
	Button b3 = Button(BT_UP,		A0_BTUP,	A0_BTUP);
	Button b4 = Button(BT_DOWN,		A0_BTDOWN,	A0_BTDOWN);
	Button b5 = Button(BT_ENTER,	A0_BTENTER,	A0_BTENTER, 5);

	// General Purpose
	int lastButtonPushed=0;  // Last button pushed

///////////////////////////////////////////////////////////////////////////////
//                         --== MENU VARIABLES ==--
// define variables & initialize default values 
///////////////////////////////////////////////////////////////////////////////

	byte current_menu=0;
	byte previous_menu=-1;

///////////////////////////////////////////////////////////////////////////////
//         --== LIGHTNING DETECTOR SYSTEM CONTROL VARIABLES ==--
// define variables & initialize default values
///////////////////////////////////////////////////////////////////////////////
	word			L_MemAdd			=2;
	LIGHTNING		L_App;
	unsigned long	L_ShutterDuration	=100;	// in ms	// HMSMS
	unsigned long	L_PostDelay			=500;	// in ms	// HMS
	int				L_Treshold			=10;
	iParam			L_ParamTreshold		={1024,0,1};
	
///////////////////////////////////////////////////////////////////////////////
//          --== CATCHER DETECTOR SYSTEM CONTROL VARIABLES ==--
// define variables & initialize default values
///////////////////////////////////////////////////////////////////////////////
	word			C_MemAdd			=12;
	CATCHER			C_App;
	unsigned long	C_PreDelay			=0;		// in ms	// HMS
	unsigned long	C_PreFocusDuration	=0;		// in ms	// SMS
	unsigned long	C_ShutterDuration	=100;	// in ms	// HMSMS
	unsigned long	C_PostDelay			=500;	// in ms	// HMS
	int				C_Treshold			=500;	// in ms
	iParam			C_ParamTreshold		={1024,0,1};
	
///////////////////////////////////////////////////////////////////////////////
//          --== INTERVALOMETER SYSTEM CONTROL VARIABLES ==--
// define variables & initialize default values
///////////////////////////////////////////////////////////////////////////////
	word			I_MemAdd			=30;
	INTERVALOMETER	I_App;
	unsigned long	I_PreDelay			=0;		// in ms	// HMS
	unsigned long	I_PreFocusDuration	=0;		// in ms	// SMS
	unsigned long	I_ShutterDuration	=100;	// in ms	// HMSMS
	unsigned long	I_ShutterInterval	=1000;	// in ms	// HMS
	int				I_ShutterMax		=10;
	iParam			I_ParamShutterMax	={1000,1,1};
	
///////////////////////////////////////////////////////////////////////////////
//                     --== GLOBAL VARIABLES ==--
// define variables & initialize default values
///////////////////////////////////////////////////////////////////////////////
	int				Photo				=0;
	int				PhotoPrev			=-1;
	hmsmsParam		HMSParam	= {HMSMS_HMS,	24,0,1,	60,0,1,	60,0,1,	1000,0,50};
	hmsmsParam		SMSParam	= {HMSMS_SMS,	24,0,1,	60,0,1,	 5,0,1,	1000,0,50};
	hmsmsParam		HMSMSParam	= {HMSMS_HMSMS,	24,0,1,	60,0,1,	60,0,1,	1000,0,50};
	byte	hh;			// hours
	byte	mm;			// minutes
	byte	ss;			// secondes
	int		ms;			// millisecondes
	byte	HMSMSpos;	// Cursor position
	int iSensor = 0;
	int iSensorPrev = 0;
	
	boolean trigger=LOW;

/*=========================================================================*/
////////////////////////////////////////////////////
// the setup routine runs once when you press reset:
////////////////////////////////////////////////////
void setup() {
	// init PWB assignment
	pinMode(ioCMD_Shutter,	OUTPUT);
	pinMode(ioCMD_PreFocus,	OUTPUT);
	pinMode(ioCMD_Flash1,	OUTPUT);
//	pinMode(ioCMD_Command1,	OUTPUT);
//	pinMode(ioCMD_Command2,	OUTPUT);
//	pinMode(ioCMD_Command3,	OUTPUT);
//	pinMode(ioCMD_Command4,	OUTPUT);
	//init default value
	digitalWrite(ioCMD_Shutter,		LOW);
	digitalWrite(ioCMD_PreFocus,	LOW);
	digitalWrite(ioCMD_Flash1,		LOW);
//	digitalWrite(ioCMD_Command1,	LOW);
//	digitalWrite(ioCMD_Command2,	LOW);
//	digitalWrite(ioCMD_Command3,	LOW);
//	digitalWrite(ioCMD_Command4,	LOW);

// Button definition
	anaBT.addButton(b1);
	anaBT.addButton(b2);
	anaBT.addButton(b3);
	anaBT.addButton(b4);
	anaBT.addButton(b5);  

	// set up the lcd's number of columns and rows: 
	lcd.begin(16, 2);
	lcd.clear();
	lcd.setCursor(0, 0); 
	lcd.print("DSLR TOOLBOX    ");
	lcd.setCursor(0, 1); 
	lcd.println("StefB. 2013 v1.0");

//	Serial.begin(9600);
//	Serial.println("LIGHTNING DETECT> StefB. 2013 v1.0");
	delay(2000);
// Load parameters from EEPROM
// if no previous backup done, default parameters are saved once then used.
	EEPROM_FirstAccess();

	trigger=LOW;
// Lightning Detector	
	L_App.SetParameter(L_ShutterDuration,L_PostDelay);
	L_Treshold  = L_Treshold;
// Catcher Detector	
	C_App.SetParameter(C_PreDelay,C_PreFocusDuration,C_ShutterDuration,C_PostDelay);
	C_Treshold  = C_Treshold; 
// Intervalometer Detector	
	I_App.SetParameter(I_PreDelay, I_PreFocusDuration, I_ShutterDuration, I_ShutterInterval, I_ShutterMax);

	current_menu=m_App_Interv;
}

////////////////////////////////////////////////////
// LOOP
////////////////////////////////////////////////////
void loop() {
	anaBT.checkButtons();
	update_menu();
}
