#include <EEPROM.h>
#include "EEPROMAnything.h"

#include "DSLR_Catcher.h" //include the declaration for this class

// CONSTANT
//const byte FOO = 13;
 
// PUBLIC
CATCHER::CATCHER(){	//<<constructor>>
	ulPreDelay=0;				//   0ms  | 04 blocks | iMemoryAddr
	ulPreFocusDuration= 0;		//   0ms  | 04 blocks | iMemoryAddr+4
	ulShutterDuration= 50;		//  50ms  | 04 blocks | iMemoryAddr+8
	ulPostDelay= 500;			// 500ms  | 04 blocks | iMemoryAddr+12
	ulStartTime=0;
	iNbShutter=0;
	iMemoryAddr=0;
	bTakeShot=LOW;
	bShutter =LOW;
	bPreFocus=LOW;
	ntt=0;	// Next Tigger Time
	tsr=0;	// Time Shutter Release

//	LoadParameters();
}
CATCHER::CATCHER(unsigned long  prd, unsigned long pfd, unsigned long sd, unsigned long pod){	//<<constructor>>
	if (pfd>prd) { prd=pfd; }
	ulPreDelay=prd;				//   0ms  | 04 blocks | iMemoryAddr
	ulPreFocusDuration= pfd;	//   0ms  | 04 blocks | iMemoryAddr+4
	ulShutterDuration= sd;		//  50ms  | 04 blocks | iMemoryAddr+8
	ulPostDelay= pod;			// 500ms  | 04 blocks | iMemoryAddr+12
	iNbShutter=0;
	iMemoryAddr=0;
	bTakeShot=LOW;
	bShutter =LOW;
	bPreFocus=LOW;
	ntt=0;	// Next Tigger Time
	tsr=0;	// Time Shutter Release

//	LoadParameters();
}
CATCHER::~CATCHER(){	//<<destructor>>
/*nothing to destruct*/
}
 
void CATCHER::Start(){
	iNbShutter= 0;
	bTakeShot = LOW;
	bShutter  = LOW;
	bPreFocus = LOW;
}
void CATCHER::Stop(){
	bTakeShot = LOW;
	bShutter  = LOW;
	bPreFocus = LOW;
}
void CATCHER::SetParameter(unsigned long prd, unsigned long pfd, unsigned long sd, unsigned long pod){

	if (pfd>prd) { prd=pfd; }
	ulPreDelay         = prd;
	ulPreFocusDuration = pfd;
	ulShutterDuration  = sd;
	ulPostDelay        = pod;

	SaveParameters();
}
void CATCHER::GetParameter(unsigned long *prd, unsigned long *pdf, unsigned long *sd, unsigned long *pod){
	LoadParameters();
	
	*prd = ulPreDelay;
	*pdf = ulPreFocusDuration;
	*sd  = ulShutterDuration;
	*pod = ulPostDelay;
}
int  CATCHER::GetNbShutter() {
	return (iNbShutter);
}
void CATCHER::SetMemoryAddress(word addr) {
	iMemoryAddr = addr;
}
int	CATCHER::GetEEPROMMemorySize(){
	return(16);	// 16blocks needed to save in EEPROM
}
void CATCHER::SetPreFocusStatus(boolean s){
	bPreFocus=s;
}
void CATCHER::SetShutterStatus(boolean s){
	bShutter=s;
}
boolean CATCHER::IsTakeShot(){
	return(bTakeShot);
}
boolean CATCHER::IsPreFocus(){
	return(bPreFocus);
}
boolean CATCHER::IsShutter(){
	return(bShutter);
}
boolean CATCHER::DoCatcher(int value,int treshold){
	unsigned long tick;
	boolean hasChanged=LOW;

	if (bTakeShot == LOW){
		if (value>=treshold) {
			bTakeShot = HIGH;
			ulStartTime = millis();
			ntt = ulPreDelay-ulPreFocusDuration;
			tsr = ulPreDelay+ulShutterDuration;
			}
	}
	if (bTakeShot==HIGH) {
		tick = millis()-ulStartTime;
		if (!bPreFocus && (tick >= ntt) && (tick<ulPreDelay)){
			bPreFocus = HIGH;
			ntt=ulPreDelay;
			hasChanged=HIGH;
		}
		if (!bShutter && (tick >= ntt) && (tick<tsr) )	{
			bShutter = HIGH;
			ntt=ntt+ulShutterDuration;
			hasChanged=HIGH;
		}
		if (bShutter && (tick >= ntt))	{
			bShutter = LOW;
			bPreFocus = LOW;
			iNbShutter++;
			ntt=ntt+ulPostDelay;
			hasChanged=HIGH;
		}
		if (!bPreFocus && (tick >= ntt))	{
			bTakeShot = LOW;
			ntt=0;
		}
	}
	return(hasChanged);
}

// PRIVATE
boolean CATCHER::LoadParameters(){
unsigned long *ultemp=0;
	if (iMemoryAddr >0){
		EEPROM_readAnything(iMemoryAddr,    *ultemp);
		ulPreDelay=*ultemp;
		EEPROM_readAnything(iMemoryAddr+4,  *ultemp);
		ulPreFocusDuration=*ultemp;
		EEPROM_readAnything(iMemoryAddr+8,  *ultemp);
		ulShutterDuration=*ultemp;
		EEPROM_readAnything(iMemoryAddr+12, *ultemp);
		ulPostDelay=*ultemp;
		return(true);
	} else { return(false);}
}
boolean CATCHER::SaveParameters(){
	if (iMemoryAddr >0){
		EEPROM_writeAnything(iMemoryAddr,    ulPreDelay);
		EEPROM_writeAnything(iMemoryAddr+4,  ulPreFocusDuration);
		EEPROM_writeAnything(iMemoryAddr+8,  ulShutterDuration);
		EEPROM_writeAnything(iMemoryAddr+12, ulPostDelay);
		return(true);
	} else { return(false);}
}
