#include <EEPROM.h>
#include "EEPROMAnything.h"

#include "DSLR_Intervalometer.h" //include the declaration for this class

// CONSTANT
//const byte FOO = 13;
 
// PUBLIC
INTERVALOMETER::INTERVALOMETER(){	//<<constructor>>
	ulStartTime=0;
	iNbShutter=0;
	iMemoryAddr=0;
	bShutter=LOW;
	bPreFocus=LOW;
	bTakeShot = LOW;
	tpfn=0;
	tssn=0;
	tesn=0;

	LoadParameters();
}
INTERVALOMETER::INTERVALOMETER(unsigned long pd, unsigned long pfd, unsigned long sd, unsigned long si, int nsm){	//<<constructor>>
	ulStartTime=0;
	ulPreDelay=pd;
	ulPreFocusDuration= pfd;
	ulShutterDuration= sd;
	ulShutterInterval= si;
	iNbShutterMax=nsm;
	iNbShutter=0;
	iMemoryAddr=0;
	bShutter=LOW;
	bPreFocus=LOW;
	bTakeShot = LOW;
	tpfn=0;
	tssn=0;
	tesn=0;

//	LoadParameters();
}
INTERVALOMETER::~INTERVALOMETER(){	//<<destructor>>
/*nothing to destruct*/
}
 
void INTERVALOMETER::Start(){
	ulStartTime = millis();
	iNbShutter=-1;
	bTakeShot = LOW;
	bPreFocus=LOW;
	bShutter=LOW;
	CheckParameters();
}
void INTERVALOMETER::Stop(){
	bTakeShot = LOW;
	bPreFocus=LOW;
	bShutter=LOW;		
}
boolean INTERVALOMETER::SetParameter(unsigned long pd, unsigned long pfd, unsigned long sd, unsigned long si, int nsm){
	ulPreDelay         = pd;
	ulPreFocusDuration = pfd;
	ulShutterDuration  = sd;
	ulShutterInterval  = si;
	iNbShutterMax      = nsm;

	CheckParameters();
}
void INTERVALOMETER::GetParameter(unsigned long &pd, unsigned long &pfd, unsigned long &sd, unsigned long &si, int &nsm){
	LoadParameters();
	
	pd  = ulPreDelay;
	pfd = ulPreFocusDuration;
	sd =  ulShutterDuration;
	si =  ulShutterInterval;
	nsm = iNbShutterMax;
}
int  INTERVALOMETER::GetNbShutter() {
	return (iNbShutter);
}
void INTERVALOMETER::SetMemoryBlock(word addr) {
	iMemoryAddr = addr;
}
int	INTERVALOMETER::GetEEPROMMemorySize(){
	return(18);	// 16blocks needed to save in EEPROM
}
void INTERVALOMETER::SetShutterStatus(boolean s){
	bShutter=s;
}
void INTERVALOMETER::SetPreFocusStatus(boolean s){
	bPreFocus=s;
}
boolean INTERVALOMETER::IsTakeShot(){
	return(bTakeShot);
}
boolean INTERVALOMETER::IsPreFocus(){
	return(bPreFocus);
}
boolean INTERVALOMETER::IsShutter(){
	return(bShutter);
}
void INTERVALOMETER::DoIntervalometer(){
	unsigned long tick;
	
	tick = millis()-ulStartTime;

	if (iNbShutter==-1){	// n0
		iNbShutter=0;
		tpfn=ulPreDelay-ulPreFocusDuration;
		tssn=ulPreDelay;
		tesn=ulPreDelay+ulShutterDuration;
	}
	if ((iNbShutter >= 0) && (iNbShutter<iNbShutterMax)){
		if (!bPreFocus && (tick >= tpfn)) {
			bPreFocus=HIGH;
			bTakeShot=HIGH;
			}
		if (!bShutter && (tick >= tssn)) {
			bShutter=HIGH;
			}
		if (bShutter && (tick >= tesn)) {
			bPreFocus = LOW;
			bShutter  = LOW;
			bTakeShot = LOW;
			iNbShutter ++;
			tpfn = tpfn + ulShutterInterval;	// n+1
			tssn = tssn + ulShutterInterval;	// n+1
			tesn = tesn + ulShutterInterval;	// n+1
		}
	}
}

// PRIVATE
boolean INTERVALOMETER::LoadParameters(){
	if (iMemoryAddr >0){
		EEPROM_readAnything(iMemoryAddr,    ulPreDelay);
		EEPROM_readAnything(iMemoryAddr+4,  ulPreFocusDuration);
		EEPROM_readAnything(iMemoryAddr+8,  ulShutterDuration);
		EEPROM_readAnything(iMemoryAddr+12, ulShutterInterval);
		EEPROM_readAnything(iMemoryAddr+16, iNbShutterMax);
		return(true);
	} else { return(false);}
}
boolean INTERVALOMETER::SaveParameters(){
	if (iMemoryAddr >0){
		EEPROM_writeAnything(iMemoryAddr,    ulPreDelay);
		EEPROM_writeAnything(iMemoryAddr+4,  ulPreFocusDuration);
		EEPROM_writeAnything(iMemoryAddr+8,  ulShutterDuration);
		EEPROM_writeAnything(iMemoryAddr+12, ulShutterInterval);
		EEPROM_writeAnything(iMemoryAddr+16, iNbShutterMax);
		return(true);
	} else { return(false);}
}
void INTERVALOMETER::CheckParameters(){
	if ( ulPreDelay <= ulPreFocusDuration ) {
		ulPreDelay=ulPreFocusDuration;
	}
		
	if ( ulShutterDuration + ulPreFocusDuration >= ulShutterInterval ) {
		ulShutterInterval = ulShutterDuration + ulPreFocusDuration;
	}
}
