#include <EEPROM.h>
#include "EEPROMAnything.h"

#include "DSLR_Lightning.h" //include the declaration for this class

// CONSTANT
//const byte FOO = 13;
 
// PUBLIC
LIGHTNING::LIGHTNING(){	//<<constructor>>
	ulShutterDuration= 50;	// 50ms   | 04 blocks | iMemoryAddr
	ulPostDelay= 500;		// 500ms  | 04 blocks | iMemoryAddr+4
	ulStartTime=0;
	iNbShutter=0;
	iMemoryAddr=0;
	bTakeShot=LOW;
	bShutter =LOW;
	bPreFocus=LOW;
	ntt=0;	// Next Tigger Time
}
LIGHTNING::LIGHTNING(unsigned long  sd, unsigned long  pod, word add){	//<<constructor>>
	ulShutterDuration= sd;
	ulPostDelay= pod;
	ulStartTime=0;
	iNbShutter=0;
	iMemoryAddr=add;
	bTakeShot=LOW;
	bShutter =LOW;
	bPreFocus=LOW;
	ntt=0;	// Next Tigger Time

//	LoadParameters();
}
LIGHTNING::~LIGHTNING(){	//<<destructor>>
/*nothing to destruct*/
}
 
void LIGHTNING::Start(){
	iNbShutter=0;
	bTakeShot=LOW;
	bPreFocus=LOW;
	bShutter=LOW;
}
void LIGHTNING::Stop(){
	bTakeShot = LOW;
	bShutter  = LOW;
	bPreFocus = LOW;
}
void LIGHTNING::SetParameter(unsigned long  sd, unsigned long  pod){
	ulShutterDuration  = sd;
	ulPostDelay        = pod;

	SaveParameters();
}
void LIGHTNING::GetParameter(unsigned long *sd, unsigned long  *pod){
	LoadParameters();
	
	*sd  = ulShutterDuration;
	*pod = ulPostDelay;
}
int  LIGHTNING::GetNbShutter() {
	return (iNbShutter);
}
void LIGHTNING::SetMemoryAddress(word addr) {
	iMemoryAddr = addr;
}
void LIGHTNING::SetShutterStatus(boolean s){
	bShutter=s;
}
void LIGHTNING::SetPreFocusStatus(boolean s){
	bPreFocus=s;
}
void LIGHTNING::SetTakeShotStatus(boolean s){
	bTakeShot=s;
}
boolean LIGHTNING::IsTakeShot(){
	return(bTakeShot);
}
boolean LIGHTNING::IsPreFocus(){
	return(bPreFocus);
}
boolean LIGHTNING::IsShutter(){
	return(bShutter);
}
boolean LIGHTNING::DoLightningDetector(int value,int treshold){
	unsigned long tick;
	boolean hasChanged=LOW;
	
	if (bTakeShot == LOW){
		if (value>=treshold) {
			bTakeShot = HIGH;
			bPreFocus = HIGH;
			bShutter  = HIGH;
			ulStartTime = millis();
			ntt = ulShutterDuration;
			hasChanged=HIGH;
		}
	}
	if (bTakeShot==HIGH) {
		tick = millis()-ulStartTime;
		if (bShutter && (tick >= ntt) )	{
			bShutter = LOW;
			bPreFocus = LOW;
			iNbShutter++;
			ntt=ntt+ulPostDelay;
			hasChanged=HIGH;
		}
		if (!bShutter && (tick >= ntt))	{
			bTakeShot = LOW;
		}
	}
	return(hasChanged);
}

// PRIVATE
boolean LIGHTNING::LoadParameters(){
unsigned long *ultemp=0;
	if (iMemoryAddr >0){
		EEPROM_readAnything(iMemoryAddr,    *ultemp);
		ulShutterDuration=*ultemp;
		EEPROM_readAnything(iMemoryAddr+4,  *ultemp);
		ulPostDelay=*ultemp;
		return(true);
	} else { return(false);}
}
boolean LIGHTNING::SaveParameters(){
	if (iMemoryAddr >0){
		EEPROM_writeAnything(iMemoryAddr,    ulShutterDuration);
		EEPROM_writeAnything(iMemoryAddr+4,  ulPostDelay);
		return(true);
	} else { return(false);}
}
